package com.ruoyi.tc.controller;


import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.tc.entity.AssetApp;
import com.ruoyi.tc.entity.AssetTask;
import com.ruoyi.tc.entity.Unit;
import com.ruoyi.tc.entity.po.AssetAppCpPo;
import com.ruoyi.tc.entity.po.AssetAppJyPo;
import com.ruoyi.tc.entity.po.AssetEmailJyPo;
import com.ruoyi.tc.entity.po.AssetMiniProgramsCpPo;
import com.ruoyi.tc.entity.request.AssetAppPageRequest;
import com.ruoyi.tc.entity.request.AssetAuditPageRequest;
import com.ruoyi.tc.entity.request.AssetAuditRequest;
import com.ruoyi.tc.entity.zztb.AssetAppZztbPo;
import com.ruoyi.tc.service.*;
import com.ruoyi.tc.zztbService.AssetAppZztbService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;

/**
 * 资产移动应用程序表(asset_app)表控制层
 *
 * @author makejava
 * @since 2024-11-28 17:13:05
 */
@Api(tags = "资产移动应用程序表控制层")
@RestController
@RequestMapping("/tc/assetApp")
public class AssetAppController extends BaseController {
    /**
     * 服务对象
     */
    @Resource
    private AssetAppService assetAppService;

    @Resource
    private AssetAppJyService assetAppJyService;

    @Resource
    private AssetTaskService assetTaskService;
    @Resource
    private AssetAppCpService assetAppCpService;

    @Resource
    private UnitService unitService;

    @Resource
    private AssetAppZztbService assetAppZztbService;

    /**
     * 分页查询
     *
     * @param req 分页对象
     * @return 查询结果
     */
    @ApiOperation(value = "分页查询")
    @GetMapping
    public AjaxResult queryByPage(AssetAppPageRequest req) {
        Page<AssetApp> page = new Page<>();
        page.setCurrent(req.getCurrent());
        page.setSize(req.getSize());
        try {
            if (!SecurityUtils.getLoginUser().getUser().isAdmin() && !SecurityUtils.hasRole("common")) {
                req.setDwmc(SecurityUtils.getLoginUser().getUser().getNickName());
            }
        } catch (Exception e) {
            throw new ServiceException("获取用户信息异常");
        }
        return success(assetAppService.page(page, req));
    }
    //=================================================

    /**
     * 单位自编辑进行校验
     */
    @ApiOperation(value = "单位自编辑进行校验")
    @PreAuthorize("@ss.hasAnyRoles('unit')")
    @PostMapping("/unitEdit")
    public AjaxResult unitEdit(@RequestBody @Valid AssetAppZztbPo assetAppZztbPo) {
        List<Unit> list = unitService.lambdaQuery().eq(Unit::getNickName, assetAppZztbPo.getSsdw()).eq(Unit::getDelFlag, 0).list();
        if (list == null || list.isEmpty()) {
            throw new ServiceException("请选择已有单位!");
        }

        AssetAppZztbPo one = assetAppZztbService.lambdaQuery().eq(AssetAppZztbPo::getAppName, assetAppZztbPo.getAppName())
                .eq(AssetAppZztbPo::getSsdw, assetAppZztbPo.getSsdw()).isNull(AssetAppZztbPo::getTaskId).one();
        assetAppZztbPo.setAuditState("1");
        assetAppZztbPo.setAuditYy(null);
        if (one != null) {
            assetAppZztbPo.setAppId(one.getAppId());
            assetAppZztbPo.setAssetId(one.getAssetId());
            assetAppZztbService.updateById(assetAppZztbPo);
        } else {
            assetAppZztbPo.setAssetId(assetAppService.lambdaQuery().eq(AssetApp::getAppName, assetAppZztbPo.getAppName())
                    .eq(AssetApp::getSsdw, assetAppZztbPo.getSsdw()).one().getId());
            assetAppZztbService.save(assetAppZztbPo);
        }
        return success();
    }

    /**
     * 分页查询管理端或者单位端审核列表
     *
     * @param as 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "分页查询管理端或者单位端审核列表", response = AssetAppZztbPo.class)
    @GetMapping("/getAuditList")
    public AjaxResult getAuditList(AssetAuditPageRequest as) {
        Page<AssetAppZztbPo> page = new Page<>();
        page.setSize(as.getSize());
        page.setCurrent(as.getCurrent());
        try {
            if (!SecurityUtils.getLoginUser().getUser().isAdmin() && !SecurityUtils.hasRole("common")) {
                as.setDwmc(SecurityUtils.getLoginUser().getUser().getNickName());
            }
        } catch (Exception e) {
            throw new ServiceException("获取用户信息异常");
        }
        return success(assetAppZztbService.getAuditList(page, as));
    }

    /**
     * 查看中间数据详情
     */
    @ApiOperation(value = "单位端查看中间数据详情", response = AssetAppJyPo.class)
    @GetMapping("/lookInfo/{id}")
    public AjaxResult lookInfo(@PathVariable Long id) {
        return success(assetAppZztbService.getById(id));
    }

    /**
     * 管理端审核
     *
     * @param as 查询实体
     * @return 所有数据
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,common')")
    @ApiOperation(value = "管理端审核")
    @PostMapping("/audit")
    public AjaxResult audit(@Valid @RequestBody AssetAuditRequest as) {
        AssetAppZztbPo ass = new AssetAppZztbPo();
        BeanUtil.copyProperties(as, ass);
        ass.setAppId(as.getCurrentId());
        assetAppZztbService.updateById(ass);
        //通过数据同步
        if ("2".equals(as.getAuditState()) && as.getXzType() != 1) {
            AssetAppJyPo byId = assetAppJyService.getById(as.getCurrentId());
            AssetApp aa = new AssetApp();
            BeanUtil.copyProperties(byId, aa);
            aa.setId(byId.getAssetId());
            //修改主表
            assetAppService.updateById(aa);
            //如果有任务也修改任务这边
            AssetAppJyPo one = assetAppJyService.lambdaQuery()
                    .isNotNull(AssetAppJyPo::getTaskId)
                    .ne(AssetAppJyPo::getStatus, 5)
                    .eq(AssetAppJyPo::getAssetId, byId.getAssetId())
                    .orderByDesc(AssetAppJyPo::getAppId)
                    .last("LIMIT 1").one();
            if (one != null) {
                //根据任务id查询任务是否关闭
                AssetTask two = assetTaskService.lambdaQuery().eq(AssetTask::getId, one.getTaskId()).eq(AssetTask::getTaskStatus, 0).one();
                if (two == null) {
                    Integer status = one.getStatus();
                    Integer taskId = one.getTaskId();
                    Long appId = one.getAppId();
                    BeanUtil.copyProperties(byId, one);
                    one.setAppId(appId);
                    one.setTaskId(taskId);
                    one.setAuditState(null);
                    one.setAuditYy(null);
                    one.setStatus(status);
                    assetAppJyService.updateById(one);
                }

            }

        }
        else if("2".equals(as.getAuditState())) {
            AssetAppZztbPo byId = assetAppZztbService.getById(as.getCurrentId());
            AssetApp aa = new AssetApp();
            BeanUtil.copyProperties(byId, aa);
            aa.setId(byId.getAssetId());
            //根据资产名称和所属单位查询主表是否有数据
            AssetApp assetApp = assetAppService.lambdaQuery().eq(AssetApp::getAppName, byId.getAppName()).eq(AssetApp::getSsdw, byId.getSsdw()).one();
            if (assetApp != null) {
                //修改主表
                if (assetApp.getXzType() == 0) {
                    BeanUtil.copyProperties(aa, assetApp, "id");
                    assetApp.setXzType(0);
                    assetApp.setDelFlag("0");

                    assetAppService.updateById(assetApp);

                }else{
                    BeanUtil.copyProperties(aa, assetApp, "id");
                    assetAppService.updateById(assetApp);
                }
                //如果有任务也修改任务这边
                AssetAppJyPo one = assetAppJyService.lambdaQuery()
                        .ne(AssetAppJyPo::getStatus, 5)
                        .eq(AssetAppJyPo::getAppName, byId.getAppName())
                        .orderByDesc(AssetAppJyPo::getAppId)
                        .last("LIMIT 1").one();
                if (one != null) {
                    //根据任务id查询任务是否关闭
                    AssetTask two = assetTaskService.lambdaQuery().eq(AssetTask::getId, one.getTaskId()).eq(AssetTask::getTaskStatus, 0).one();
                    if (two == null) {
                        Integer status = one.getStatus();
                        Integer taskId = one.getTaskId();
                        Long appId = one.getAppId();
                        BeanUtil.copyProperties(byId, one);
                        one.setAppId(appId);
                        one.setTaskId(taskId);
                        one.setAuditState(null);
                        one.setAuditYy(null);
                        one.setStatus(status);
                        assetAppJyService.updateById(one);
                    }

                }
            } else {
                //新增主表
                assetAppService.save(aa);
                //查询改单位下是否有进行中的任务
                AssetTask task= assetTaskService.lambdaQuery().like(AssetTask::getDwmc,aa.getSsdw()).like(AssetTask::getType,"1").eq(AssetTask::getTaskStatus,1).one();
                if (task!=null){
                    AssetAppCpPo bb = new AssetAppCpPo();
                    BeanUtil.copyProperties(byId, bb);
                    bb.setAssetId(aa.getId());
                    bb.setTaskId(task.getId());
                    bb.setAppId(null);
                    assetAppCpService.save(bb);
                }
            }


        }
        else if  ("3".equals(as.getAuditState())){
            AssetAppZztbPo byId = assetAppZztbService.getById(as.getCurrentId());
            AssetApp aa = new AssetApp();
            BeanUtil.copyProperties(byId, aa);
            aa.setId(byId.getAssetId());
            //根据资产名称和所属单位查询主表是否有数据
            AssetApp assetApp = assetAppService.lambdaQuery().eq(AssetApp::getAppName, byId.getAppName()).eq(AssetApp::getSsdw, byId.getSsdw()).one();
            if (assetApp != null) {
                //修改主表
                if (assetApp.getXzType() == 0) {
                    BeanUtil.copyProperties(aa, assetApp, "id");
                    assetApp.setXzType(0);
                    assetApp.setDelFlag("1");
                    assetAppService.updateById(assetApp);

                } else {
                    BeanUtil.copyProperties(aa, assetApp, "id");
                    assetApp.setDelFlag("1");
                    assetAppService.updateById(assetApp);
                }
            }
        }
        return success();
    }
    //=================================================

    /**
     * 通过主键查询单条app数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ApiOperation(value = "通过主键查询单条app数据")
    @GetMapping("/{id}")
    public AjaxResult queryById(@PathVariable("id") Long id) {
        return success(assetAppService.getById(id));
    }

    /**
     * 新增数据
     *
     * @param assetApp 实体
     * @return 新增结果
     */
    @ApiOperation(value = "新增数据")
    @PostMapping
    @Log(title = "新增app数据", businessType = BusinessType.INSERT)
    public AjaxResult add(@RequestBody AssetApp assetApp) {

        return success(assetAppService.add(assetApp));
    }

    /**
     * 编辑数据
     *
     * @param assetApp 实体
     * @return 编辑结果
     */
    @ApiOperation(value = "编辑数据")
    @Log(title = "编辑app数据", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody AssetApp assetApp) {

        return success(assetAppService.edit(assetApp));
    }

    /**
     * 删除数据
     *
     * @param id 主键
     * @return 删除是否成功
     */
    @ApiOperation(value = "删除数据")
    @Log(title = "删除app数据", businessType = BusinessType.DELETE)
    @DeleteMapping("/delete/{id}")
    public AjaxResult deleteById(@PathVariable Long id) {
        assetAppService.deleteById(id);
        return success();
    }

    /**
     * 下载资产导入模板
     */
    @ApiOperation(value = "下载App导入模板")
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response) {
        ExcelUtil<AssetApp> util = new ExcelUtil<>(AssetApp.class);
        util.importTemplateExcel(response, "App导入模板");
    }

    /**
     * app导出
     */
    @Transactional(rollbackFor = Exception.class)
    @ApiOperation(value = "app导出")
    @Log(title = "app导出", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, AssetAppPageRequest unit) {
        List<AssetApp> list = assetAppService.getAllList(unit);
        ExcelUtil<AssetApp> util = new ExcelUtil<>(AssetApp.class);
        util.exportExcel(response, list, "app数据");
    }

    /**
     * app导入
     */
    @Transactional(rollbackFor = Exception.class)
    @ApiOperation(value = "app导入")
    @Log(title = "app导入", businessType = BusinessType.IMPORT)
    @PostMapping("/importData")
    public AjaxResult importData(MultipartFile file) throws Exception {
        ExcelUtil<AssetApp> util = new ExcelUtil<>(AssetApp.class);
        List<AssetApp> list = util.importExcel(file.getInputStream());
        if (list != null) {
            list.forEach(x -> {
                AssetApp one = assetAppService.lambdaQuery().eq(AssetApp::getSsdw, x.getSsdw())
                        .eq(AssetApp::getAppName, x.getAppName()).one();
                if (one != null) {
                    x.setId(one.getId());
                    assetAppService.edit(x);
                } else {
                    assetAppService.add(x);
                }
            });
        } else {
            throw new ServiceException("app导入数据不能为空!");
        }
        return AjaxResult.success("导入成功");
    }
}

